unit SXMathEval;

////////////////////////////////////////////////////////////////////////////////
// SXSkinComponents: Skinnable Visual Controls with Alpha Channel             //
//----------------------------------------------------------------------------//
// Version: 1.0.1                                                             //
// Author: Alexey Sadovnikov                                                  //
// Web Site: http://www.saarixx.info                                          //
// E-Mail: mail@saarixx.info                                                  //
//----------------------------------------------------------------------------//
// LICENSE:                                                                   //
// 1. You can freely distribute this file.                                    //
// 2. You can not make any changes to this file.                              //
// 3. The only person who can change this file is Alexey Sadovnikov.          //
// 4. You can use this file in your freeware projects.                        //
// 5. If you want to use this file in your shareware project, you should      //
//    purchase a project license or a personal license of SXSkinComponents.   //
// 6. You can freely use, distribute and modify skins for SXSkinComponents.   //
// 7. You can create skins for SXSkinComponents.                              //
//----------------------------------------------------------------------------//
// Copyright 2006 by Alexey Sadovnikov. All Rights Reserved.                  //
////////////////////////////////////////////////////////////////////////////////

interface

{$I Compilers.inc}

uses Windows, SysUtils, Math;

type

 TSXOnGetVariable=function(const VarName:String;var Error:Boolean):Double of object;

function SXEvalMathString(const S:String;OnGetVar:TSXOnGetVariable;var Error:Boolean):Double;

implementation

function EvalNum(S:String;OnGetVar:TSXOnGetVariable;var Error:Boolean):Double;
begin
 if (S<>'') and (S[1]='~') then S[1]:='-';
 Error:=False;
 if TryStrToFloat(S,Result) then exit;
 Result:=OnGetVar(S,Error);
end;

function EvalSqrt(const S:String;OnGetVar:TSXOnGetVariable;var Error:Boolean):Double;
begin
 if (S<>'') and (S[1]='!') then
  begin
   Result:=EvalSqrt(Copy(S,2,MaxInt),OnGetVar,Error);
   if Error then
    begin
     Result:=0;
     exit;
    end;
   Result:=Sqrt(Result);
  end else Result:=EvalNum(S,OnGetVar,Error);
end;

function EvalPower(const S:String;OnGetVar:TSXOnGetVariable;var Error:Boolean):Double;
var  A:Integer;
 R1,R2:Double;
begin
 A:=length(S);
 while (A>=1) and not (S[A]='^') do Dec(A);
 if A=0 then
  begin
   Result:=EvalSqrt(S,OnGetVar,Error);
   exit;
  end;
 R1:=EvalPower(Copy(S,1,A-1),OnGetVar,Error);
 if Error then
  begin
   Result:=0;
   exit;
  end;
 R2:=EvalSqrt(Copy(S,A+1,MaxInt),OnGetVar,Error);
 if Error then
  begin
   Result:=0;
   exit;
  end;
 Result:=Power(R1,R2);
end;

function EvalMod(const S:String;OnGetVar:TSXOnGetVariable;var Error:Boolean):Double;
var  A:Integer;
 R1,R2:Double;
begin
 A:=length(S);
 while (A>=1) and not (S[A]='%') do Dec(A);
 if A=0 then
  begin
   Result:=EvalPower(S,OnGetVar,Error);
   exit;
  end;
 R1:=EvalMod(Copy(S,1,A-1),OnGetVar,Error);
 if Error then
  begin
   Result:=0;
   exit;
  end;
 R2:=EvalPower(Copy(S,A+1,MaxInt),OnGetVar,Error);
 if Error then
  begin
   Result:=0;
   exit;
  end;
 if round(R2)=0 then
  begin
   Error:=True;
   Result:=0;
   exit;
  end;
 try
  Result:=round(R1) mod round(R2);
 except
  Result:=0;
  Error:=True;
 end;
end;

function EvalMulDiv(const S:String;OnGetVar:TSXOnGetVariable;var Error:Boolean):Double;
var  A:Integer;
   Mul:Boolean;
  RDiv:Boolean;
 R1,R2:Double;
begin
 A:=length(S);
 while (A>=1) and not (S[A] in ['*','/','|']) do Dec(A);
 if A=0 then
  begin
   Result:=EvalMod(S,OnGetVar,Error);
   exit;
  end;
 Mul:=S[A]='*';
 RDiv:=S[A]='|';
 R1:=EvalMulDiv(Copy(S,1,A-1),OnGetVar,Error);
 if Error then
  begin
   Result:=0;
   exit;
  end;
 R2:=EvalMod(Copy(S,A+1,MaxInt),OnGetVar,Error);
 if Error then
  begin
   Result:=0;
   exit;
  end;
 if Mul then Result:=R1*R2 else
  if RDiv then
   begin
    if round(R2)=0 then
     begin
      Error:=True;
      Result:=0;
      exit;
     end;
    try
     Result:=round(R1) div round(R2);
    except
     Result:=0;
     Error:=True;
    end;
   end else
    begin
     if R2=0 then
      begin
       Error:=True;
       Result:=0;
       exit;
      end;
     try
      Result:=R1/R2;
     except
      Result:=0;
      Error:=True;
     end;
    end;
end;

function EvalPlusMinus(const S:String;OnGetVar:TSXOnGetVariable;var Error:Boolean):Double;
var  A:Integer;
  Diff:Boolean;
 R1,R2:Double;
begin
 if (S<>'') and (S[1]='-') then
  begin
   Result:=EvalPlusMinus('0'+S,OnGetVar,Error);
   exit;
  end;
 A:=length(S);
 while (A>=1) and not (S[A] in ['+','-']) do Dec(A);
 if A=0 then
  begin
   Result:=EvalMulDiv(S,OnGetVar,Error);
   exit;
  end;
 Diff:=S[A]='-';
 R1:=EvalPlusMinus(Copy(S,1,A-1),OnGetVar,Error);
 if Error then
  begin
   Result:=0;
   exit;
  end;
 R2:=EvalMulDiv(Copy(S,A+1,MaxInt),OnGetVar,Error);
 if Error then
  begin
   Result:=0;
   exit;
  end;
 try
  if Diff then Result:=R1-R2 else
   Result:=R1+R2;
 except
  Result:=0;
  Error:=True;
 end;
end;

function EvalAllParenth(S:String;OnGetVar:TSXOnGetVariable;var Error:Boolean):Double;
var A,B,C:Integer;
begin
 repeat
  A:=Pos('(',S);
  if A>0 then
   begin
    C:=1;
    B:=A;
    repeat
     Inc(B);
     if B<=length(S) then
      begin
       case S[B] of
        '(': Inc(C);
        ')': Dec(C);
       end;
      end;
    until (B>length(S)) or (C=0);
    if C>0 then
     begin
      Result:=0;
      Error:=True;
      exit;
     end;
    S:=Copy(S,1,A-1)+FloatToStr(EvalAllParenth(Copy(S,A+1,B-A-1),OnGetVar,Error))+Copy(S,B+1,MaxInt);
    if (length(S)>=A) and (S[A]='-') then S[A]:='~';
    if Error then
     begin
      Result:=0;
      exit;
     end;
   end;
 until A=0;
 Result:=EvalPlusMinus(S,OnGetVar,Error);
end;

function SXEvalMathString(const S:String;OnGetVar:TSXOnGetVariable;var Error:Boolean):Double;
var A:Integer;
begin
 for A:=1 to length(S) do
  if S[A] in ['!','-','+','*','/','|','%','(',')','^'] then
   begin
    Result:=EvalAllParenth(S,OnGetVar,Error);
    exit;
   end;
 Result:=EvalNum(S,OnGetVar,Error);
end;

end.
